/*
 * Main program.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "common.h"
#include "config.h"
#include "imports.h"
#include "summary.h"
#include "templates.h"
#include "version.h"



/* Option definitions. */

static struct option long_options[] = {
/* long opt       following           var pointer short opt */
  {"all",         no_argument,        NULL,       'a'       },
  {"client",      no_argument,        NULL,       'c'       },
  {"dir",         required_argument,  NULL,       'd'       },
  {"files",       no_argument,        NULL,       'f'       },
  {"headers",     no_argument,        NULL,       'h'       },
  {"help",        no_argument,        NULL,       '?'       },
  {"interfaces",  no_argument,        NULL,       'i'       },
  {"language",    required_argument,  NULL,       'l'       },
  {"recursive",   no_argument,        NULL,       'R'       },
  {"routines",    no_argument,        NULL,       'r'       },
  {"server",      no_argument,        NULL,       's'       },
  {"verbose",     no_argument,        NULL,       'v'       },
  {"version",     no_argument,        NULL,       'V'       },
  {0,             0,                  0,          0         },
  };


/* Main program, parsing indicated files or standard input. */

int main(int argc, char *argv[])
{
  char *arg, *progname;
  int option, index, selected_content = 0, selected_role = 0;
  struct interface *iface;

#if YYDEBUG == 1
  yydebug = 0;
#endif

  /* The configuration structure is already initialised in config.c. */

  /* Accept various options. */

  while (1)
  {
    option = getopt_long(argc, argv, "acd:fhil:rRsvV?", long_options, NULL);
    if (option == -1)
      break;

    switch (option)
    {
      /* Generate everything. */
      case 'a':
        conf.client = 1; conf.server = 1;
        conf.headers = 1; conf.interfaces = 1; conf.routines = 1;
        selected_content = 1; 
        selected_role = 1; 
        break;

      /* Select client output. */
      case 'c':
        conf.client = 1;
        selected_role = 1;
        break;

      /* Set output directory. */
      case 'd':
        conf.output_dir = optarg;
        break;

      /* Only report processed file basenames. */
      case 'f':
        conf.show_filenames = 1;
        break;

      /* Generate headers. */
      case 'h':
        conf.headers = 1;
        selected_content = 1;
        break;

      /* Generate interface headers. */
      case 'i':
        conf.interfaces = 1;
        selected_content = 1;
        break;

      /* Select output language. */
      case 'l':
        if (!strcasecmp(optarg, "c"))
          conf.language = C_LANGUAGE;
        else if (!strcasecmp(optarg, "c++"))
          conf.language = CPP_LANGUAGE;
        else
        {
          fprintf(stderr, "Output language not recognised: %s\n", optarg);
          return 1;
        }
        break;

      /* Generate routines (definitions). */
      case 'r':
        conf.routines = 1;
        selected_content = 1;
        break;

      /* Also generate files associated with base interfaces. */
      case 'R':
        conf.generate_all = 1;
        break;

      /* Select server output. */
      case 's':
        conf.server = 1;
        selected_role = 1;
        break;

      /* Verbose reporting. */
      case 'v':
        conf.verbose = 1;
        break;

      /* Show version (defined in the Makefile) and exit. */
      case 'V':
        puts(VERSION_INFORMATION);
        return 0;

      /* Show help and exit. */
      case '?':
        progname = make_basename(argv[0]);
        printf(help_text, progname);
        free(progname);
        return 0;

      default:
        return 1;
    }
  }

  /* Without any output selection, enable all output of the given kind. */

  if (!selected_content)
  {
    conf.headers = 1; conf.interfaces = 1; conf.routines = 1;
  }

  if (!selected_role)
  {
    conf.client = 1; conf.server = 1;
  }

  /* Produce an error without any input files. */

  if (optind >= argc)
  {
    fputs("No input files.\n", stderr);
    return 1;
  }

  /* Process named files. */

  for (index = optind; index < argc; index++)
  {
    arg = argv[index];

    /* Import each file for processing. */

    iface = import_file(arg, 1);

    if (iface == NULL)
    {
      fprintf(stderr, "Could not process file: %s\n", arg);
      return 1;
    }

    /* Summarise interfaces if requested. */

    if (conf.verbose)
      show_interfaces(iface);
  }

  return 0;
}
